@@ -100,6 +100,7 @@ typedef enum {
   LOGF_Keys,
   LOGF_Logging,
   LOGF_Nameserv,
+  LOGF_PrivOps,
   LOGF_Rtc,
   LOGF_Regress,
   LOGF_Sys,
@@ -108,7 +109,6 @@ typedef enum {
   LOGF_SysMacOSX,
   LOGF_SysNetBSD,
   LOGF_SysSolaris,
-  LOGF_SysSunOS,
   LOGF_SysTimex,
   LOGF_SysWinnt,
   LOGF_TempComp,
